package com.engine.organization.service.impl;

import com.engine.core.impl.Service;
import com.engine.organization.entity.extend.ExtendInfoOperateType;
import com.engine.organization.entity.extend.bo.ExtendGroupBO;
import com.engine.organization.entity.extend.param.ExtendInfoParams;
import com.engine.organization.entity.extend.po.ExtendGroupPO;
import com.engine.organization.entity.extend.po.ExtendInfoPO;
import com.engine.organization.entity.extend.po.ExtendTitlePO;
import com.engine.organization.entity.personnelcard.*;
import com.engine.organization.mapper.extend.ExtMapper;
import com.engine.organization.mapper.extend.ExtendGroupMapper;
import com.engine.organization.mapper.extend.ExtendInfoMapper;
import com.engine.organization.mapper.extend.ExtendTitleMapper;
import com.engine.organization.mapper.personnelcard.PersonnelCardMapper;
import com.engine.organization.service.HrmPersonnelCardService;
import com.engine.organization.util.OrganizationAssert;
import com.engine.organization.util.db.MapperProxyFactory;
import com.engine.organization.util.field.FieldDefinedValueUtil;
import com.engine.portal.biz.constants.ModuleConstants;
import com.engine.portal.biz.nonstandardfunction.SysModuleInfoBiz;
import weaver.blog.BlogShareManager;
import weaver.cowork.CoworkShareManager;
import weaver.crm.CrmShareBase;
import weaver.docs.search.DocSearchComInfo;
import weaver.general.Util;
import weaver.workflow.search.WorkflowRequestUtil;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @author:dxfeng
 * @createTime: 2022/10/08
 * @version: 1.0
 */
public class HrmPersonnelCardServiceImpl extends Service implements HrmPersonnelCardService {
    private static final String IS_SHOW = "1";

    private PersonnelCardMapper getPersonnelCardMapper() {
        return MapperProxyFactory.getProxy(PersonnelCardMapper.class);
    }

    private ExtendGroupMapper getExtendGroupMapper() {
        return MapperProxyFactory.getProxy(ExtendGroupMapper.class);
    }

    private ExtendTitleMapper getExtendTitleMapper() {
        return MapperProxyFactory.getProxy(ExtendTitleMapper.class);
    }

    private ExtendInfoMapper getExtendInfoMapper() {
        return MapperProxyFactory.getProxy(ExtendInfoMapper.class);
    }

    private ExtMapper getExtMapper() {
        return MapperProxyFactory.getProxy(ExtMapper.class);
    }


    @Override
    public Map<String, Object> getPersonnelCard(Long uId) {
        Map<String, Object> resultMap = new HashMap<>();
        User userInfo;
        //TODO 人员表之前关联关系，处理UID
        if (null == uId) {
            uId = (long) user.getUID();
        }
        userInfo = getPersonnelCardMapper().getUserById(uId);

        OrganizationAssert.notNull(uId, "未找到对应数据");
        // 人员基本信息

        // 统计报表
        Statistical statistical = getStatistical(uId.toString());
        List<ExtendGroupPO> extendGroupList = getExtendGroupMapper().listByType(4, IS_SHOW);
        // 获取前三个模块的信息
        List<FormItem> formItemList = new ArrayList<>();
        for (int i = 0; i < (Math.min(extendGroupList.size(), 3)); i++) {
            FormItem formItem = getFormItem(extendGroupList.get(i), uId);
            formItemList.add(formItem);
        }
        // 所有锚点
        List<Anchor> anchorList = new ArrayList<>();
        // 添加字段分组信息
        for (ExtendGroupPO extendGroupPO : extendGroupList) {
            anchorList.add(Anchor.builder().id("group_" + extendGroupPO.getId()).title(extendGroupPO.getGroupName()).build());
        }
        // 添加拓展页面信息
        List<ResourceBaseTab> resourceBaseTabList = getPersonnelCardMapper().getResourceBaseTabList();
        for (ResourceBaseTab resourceBaseTab : resourceBaseTabList) {
            anchorList.add(Anchor.builder().id("tab_" + resourceBaseTab.getId()).title(resourceBaseTab.getGroupName()).build());
        }

        resultMap.put("user", userInfo);
        resultMap.put("statistical", statistical);
        resultMap.put("formItems", formItemList);
        resultMap.put("anchorList", anchorList);
        return resultMap;
    }

    @Override
    public List<Iframe> getIframeList() {
        List<Iframe> iframeList = new ArrayList<>();
        List<ResourceBaseTab> resourceBaseTabList = getPersonnelCardMapper().getResourceBaseTabList();
        for (ResourceBaseTab resourceBaseTab : resourceBaseTabList) {
            iframeList.add(Iframe.builder().id("tab_" + resourceBaseTab.getId()).title(resourceBaseTab.getGroupName()).url(resourceBaseTab.getLinkUrl()).build());
        }
        return iframeList;
    }


    /**
     * 获取报表统计数据
     *
     * @param resourceId
     * @return
     */
    private Statistical getStatistical(String resourceId) {
        try {
            Statistical statistical = new Statistical();
            if (SysModuleInfoBiz.checkModuleStatus(ModuleConstants.Workflow)) {
                statistical.setWorkflowCount(new WorkflowRequestUtil().getRequestCount(user, resourceId));
            } else {
                statistical.setWorkflowCount(0);
            }
            if (SysModuleInfoBiz.checkModuleStatus(ModuleConstants.Doc)) {
                Object[] docCount4Hrm = new DocSearchComInfo().getDocCount4Hrm(resourceId, user);
                statistical.setDocCount(null == docCount4Hrm[1] ? 0 : Integer.parseInt(Util.null2String(docCount4Hrm[1])));
            } else {
                statistical.setDocCount(0);
            }
            if (SysModuleInfoBiz.checkModuleStatus(ModuleConstants.Crm)) {
                String[] crmCount4Hrm = new CrmShareBase().getCrmCount4Hrm(resourceId, resourceId);
                statistical.setCusCount(null == crmCount4Hrm[1] ? 0 : Integer.parseInt(Util.null2String(crmCount4Hrm[1])));
            } else {
                statistical.setCusCount(0);
            }
            if (SysModuleInfoBiz.checkModuleStatus(ModuleConstants.Cowork)) {
                String[] coworkCount4Hrm = new CoworkShareManager().getCoworkCount4Hrm(resourceId, resourceId);
                statistical.setCollaborationCount(null == coworkCount4Hrm[1] ? 0 : Integer.parseInt(Util.null2String(coworkCount4Hrm[1])));
            } else {
                statistical.setCollaborationCount(0);
            }
            if (SysModuleInfoBiz.checkModuleStatus(ModuleConstants.Blog)) {
                String[] blogCount4Hrm = new BlogShareManager().getBlogCount4Hrm(resourceId);
                statistical.setWeiboCount(null == blogCount4Hrm[1] ? 0 : Integer.parseInt(Util.null2String(blogCount4Hrm[1])));
            } else {
                statistical.setWeiboCount(0);
            }
            return statistical;
        } catch (Exception e) {
            throw new RuntimeException(e);
        }
    }

    private FormItem getFormItem(ExtendGroupPO extendGroup, Long uId) {
        FormItem formItem = new FormItem();
        formItem.setId("group_" + extendGroup.getId());
        formItem.setTitle(extendGroup.getGroupName());
        List<ExtendTitlePO> extendTitleList = getExtendTitleMapper().getTitlesByGroupID(extendGroup.getId(), IS_SHOW);
        List<Item> itemList = new ArrayList<>();
        for (ExtendTitlePO extendTitlePO : extendTitleList) {
            List<ExtendInfoPO> extendInfoList = getExtendInfoMapper().listFields(extendGroup.getExtendType().toString(), extendTitlePO.getId().toString(), ExtendGroupBO.getTableNameByGroupPO(extendGroup), ExtendInfoOperateType.LIST.getValue(), IS_SHOW);
            String fields = extendInfoList.stream().map(ExtendInfoPO::getFieldName).collect(Collectors.joining(","));
            ExtendInfoParams infoParams = ExtendInfoParams.builder().tableName(ExtendGroupBO.getTableNameByGroupPO(extendGroup)).fields(fields).params(null).id(uId).build();
            Map<String, Object> fieldValueMap = getExtMapper().listExt(infoParams);
            for (ExtendInfoPO extendInfoPO : extendInfoList) {
                Object fieldValue = fieldValueMap.get(extendInfoPO.getFieldName());
                String fieldShowName = FieldDefinedValueUtil.getFieldValue(user, extendInfoPO, Util.null2String(fieldValue));
                itemList.add(Item.builder().fieldId(extendInfoPO.getFieldNameDesc()).fieldValue(fieldShowName).build());
            }
        }
        formItem.setItems(itemList);
        return formItem;
    }


}
